package com.emsoft.data.hibernate

import com.emsoft.common.Assert
import org.hibernate.Session
import org.hibernate.SessionFactory
import org.hibernate.cfg.Configuration

class HibernateFactory {

    public static final String DEFAULT_NAME = "default"

    private static Map<String, SessionFactory> factories = [:]

    static SessionFactory initialize(String resource = DEFAULT_NAME) {
        SessionFactory factory = factories[resource]
        if (factory == null) {
            factory = (resource == DEFAULT_NAME ?
                new Configuration().configure().buildSessionFactory() :
                new Configuration().configure(resource).buildSessionFactory())
            factories.put(resource, factory)
        }
        return factory
    }

    static SessionFactory initialize(String name, Properties properties) {
        Assert.hasText(name, "name")
        Assert.notNull(properties, "properties")
        SessionFactory factory = factories[name]
        if (factory == null) {
            factory = new Configuration().setProperties(properties).buildSessionFactory()
            factories.put(name, factory)
        }
        return factory
    }

    static SessionFactory close(String name) {
        Assert.hasText(name, "name")
        SessionFactory factory = factories[name]
        if (factory != null) {
            factory.close()
            factories.remove(name)
        }
        return factory
    }

    static void closeAll() {
        factories.values()*.close()
        factories.clear()
    }

    static SessionFactory getFactory(String name = DEFAULT_NAME) {
        Assert.hasText(name, "name")
        SessionFactory factory = factories[name]
        if (factory == null)
            factory = initialize(name)
        return factory
    }

    static Session openSession(String name = DEFAULT_NAME) {
        return getFactory(name).openSession()
    }
}